/*

ere.h - Interface to Extended Regular Expression module

Beware: The matching routine is highly recursive and can require around 20
to 30 bytes per character in the source string, to match it. Thus you should
try to limit the length of the source string and/or allow a stack size of
at least 20 (to 30) * max string length. In addition, add 2KB to allow for
the use of several nested sub-expressions in the matching.

If ERECF_TOLOWER is passed to ere_compile, every character (or range of
characters) to be matched are stored in the compiled ERE in lower case.
Therefore, if strings to be matched are passed in lower case also, the result
is a case-insensitive match.

If EREMF_SHORTEST is passed to any of the ere_match functions, the code looks
for the shortest match, instead of the longest match.

If EREMF_ANY is passed to any of the ere_match functions, the code doesn't
try to find the longest (or shortest) match, it will return with the first
match it finds (which could be of any length). This can speed up matching.

*/

#ifndef ERE_H
#define	ERE_H

typedef	int ERE_BOOLEAN;
#define	ERE_TRUE  1
#define	ERE_FALSE 0

#define	EREE_OK			0
#define	EREE_MEM		1
#define	EREE_UNF_SUB		2
#define	EREE_UNEX_RANGE		3
#define	EREE_UNF_RANGE		4
#define	EREE_UNF_CCLASS		5
#define	EREE_UNEX_RSQR		6
#define	EREE_UNEX_RPAR		7
#define	EREE_UNEX_QUERY		8
#define	EREE_UNEX_PLUS		9
#define	EREE_UNEX_STAR		10
#define	EREE_UNEX_LCUR		11
#define	EREE_UNEX_RCUR		12
#define	EREE_BAD_CREP_M		13
#define	EREE_BAD_CREP_N		14
#define	EREE_UNF_CREP		15
#define	EREE_BAD_CREP		16
#define	EREE_TOO_MANY_SUB	17
#define	EREE_COMPILE_FSM	18
#define	EREE_POSIX_COLLATING	19
#define	EREE_POSIX_EQUIVALENCE	20
#define	EREE_POSIX_CCLASS_BAD	21
#define	EREE_BAD_BACKSLASH	22
#define	EREE_BAD_BACKREF	23
#define	EREE_SUBS_LEN		24
#define	EREE_EBCDIC_RANGE       25

#define	ERECF_TOLOWER		0x01

#define	EREMF_SHORTEST		0x01
#define	EREMF_ANY		0x02

#define	MAX_SPANS 9

typedef struct { int pos, len; } ERE_SPAN;

typedef struct
	{
	int      n_spans;
	ERE_SPAN spans[MAX_SPANS];
	} ERE_MATCHINFO;

#ifndef ERE_C

typedef void ERE;

extern const char *ere_error(int rc);

extern ERE *ere_compile(
	const char *str,
	int erecf,
	int *rc
	);

extern int ere_min_len(const ERE *ere);

/* Returns the number of characters in the match, starting from pos characters
   into the string to be searched. Details of sub-matches can also be returend.
   Returns -1 if no match. */
extern int ere_match(
	const ERE *ere,
	int eremf,
	const char *str, int pos,
	ERE_MATCHINFO *mi		/* can be NULL */
	);

/* Returns ERE_TRUE if a match can be found starting pos characters into the string
   to be searched. Position and length of match are returned. Details of
   sub-matches can also be returned. */
extern ERE_BOOLEAN ere_match_fwd(
	const ERE *ere, 
	int eremf,
	const char *str, int pos,
	int *pos_match, int *len_match,
	ERE_MATCHINFO *mi		/* can be NULL */
	);

/* Returns ERE_TRUE if a match can be found starting pos characters into the string
   to be searched and scanning backwards. Position and length of match are
   returned. Details of sub-matches can also be returned. */
extern ERE_BOOLEAN ere_match_bwd(
	const ERE *ere, 
	int eremf,
	const char *str, int pos,
	int *pos_match, int *len_match,
	ERE_MATCHINFO *mi		/* can be NULL */
	);

extern void ere_discard(ERE *ere);

extern ERE_BOOLEAN ere_subs_with(
	const char *str,		/* Original string searched          */
	int pos, int len,		/* Span of the entire match          */
	ERE_MATCHINFO *mi,		/* Details of sub-spans of match     */
	const char *with,		/* Specification of replacement      */
	char *out,			/* Substituted string buffer         */
	int len_out,			/* How much room in output buffer    */
	int *rc				/* Error, if ERE_FALSE returned          */
	);

#endif

#endif
